/*
** Author: dimis
** email : diochnos AT gmail DOT com
**
** File Name: math_toolbox.cpp
**
** skills (version 1.0)
** Date Started :  7 September 2009.
** Date Finished: 14 September 2009.
**
** skills (version 2.0)
** Date Started : 15 September 2009.
** Date Finished: 16 September 2009.
**
**
** Tab space: 3
**
**
** The implementation of the mathematical toolbox.
**
*/

#include <iostream>
#include <iomanip>
#include "mighty_classes.h"
#include "math_toolbox.h"

extern unsigned char Skills_Alphabetic_to_Game_Index [];
extern unsigned char Skills_Game_Index_to_Alphabetic [];

/*****************************************************************************/

MathToolbox::MathToolbox ()
{
	printing_method = (unsigned char) FANCY;	// By default, FANCY is used.

	initializeOccurrences ();
}

MathToolbox::~MathToolbox () {}

/*****************************************************************************/

/*****************************************\
* SETTING AND GETTING THE PRINTING METHOD *
\*****************************************/

unsigned char			MathToolbox::getPrintingMethod () const {	return	printing_method; }

// Set a different PrintingMethod.
bool	MathToolbox::setPrintingMethod (const unsigned char method)
{
	if (method <= (unsigned char) MAX_PRINTING_METHOD) {
		printing_method = method;
		return true;
	}
	else {
		// Invalid method given.
		// No changes made. Returning false.
		return false;
	}
}

/*****************************************************************************/

// Manipulating 'occurrences' array.
bool 						MathToolbox::incrementOccurrences (const unsigned char skill)
{
	if (skill >= DIFF_SEC_SKILLS)
		return false;	// Indicate error.

	occurrences[skill]++;

	return 	true; 	// Indicate correct behaviour.
}

/*
// Use with CAUTION the very next function.
// At the moment I only use it to merge results from the various threads!
bool 						MathToolbox::setOccurrences (const unsigned char skill, const long unsigned int number)
{
	if (skill >= DIFF_SEC_SKILLS)
		return false;	// Indicate error.

	occurrences[skill] = number;

	return 	true; 	// Indicate correct behaviour.
}
*/

long unsigned int 	MathToolbox::getOccurrences (const unsigned char skill) const
{
	if (skill >= DIFF_SEC_SKILLS)
		return 0;

	return 	occurrences[skill];
}

/*****************************************************************************/

void 						MathToolbox::initializeOccurrences ()
{
	for (int i = 0; i < DIFF_SEC_SKILLS; i++)
		occurrences [i] = 0;
}

/*****************************************************************************/

void 						MathToolbox::setProbabilities ()
{
	for (int i = 0; i < DIFF_SEC_SKILLS; i++)
		probabilities [i] = (double) ((double) occurrences [i]) / ((double) DIFF_SKILL_TREES);
}

/*****************************************************************************/

/*********************\
* PRINTING STATISTICS *
\*********************/

void 	MathToolbox::printStatistics (HERO_PTR yah, const clock_t start, const clock_t finish)
{
	setProbabilities ();
	
	if (printing_method == DEPRECATED)
		printStatisticsDeprecated (yah, start, finish);
	else if (printing_method == NEAT)
		printStatisticsNeat (yah, start, finish);
	else
		printStatisticsFancy (yah, start, finish);
}

void 	MathToolbox::printStatistics (HERO_PTR yah, TIME_DIFF * cputime)
{
	setProbabilities ();
	
	if (printing_method == DEPRECATED)
		printStatisticsDeprecated (yah, cputime);
	else if (printing_method == GODEPRECATED)
		printStatisticsGODeprecated (yah, cputime);
	else if (printing_method == NEAT)
		printStatisticsNeat (yah, cputime);
	else if (printing_method == GONEAT)
		printStatisticsGONeat (yah, cputime);
	else if (printing_method == GOFANCY)
		printStatisticsGOFancy (yah, cputime);
	else
		printStatisticsFancy (yah, cputime);
}

/*****************************************************************************/

// Private functions called by PrintStatistics.
void MathToolbox::printStatisticsDeprecated (HERO_PTR yah, const clock_t start, const clock_t finish)
{
	unsigned char pointer;

	yah->print_Name ();
	for (pointer = 1; pointer < (unsigned char) DIFF_SEC_SKILLS; pointer++)
		std::cout << (char *) get_skill_name_12 (pointer) << ": " << std::setprecision (4) << (double) (100.0 * probabilities [pointer]) << " %." << std::endl;
	std::cout << std::endl;

	if ((double) (finish - start) / CLOCKS_PER_SEC >= 0.01)
		std::cout << "Duration    : " << std::setprecision (2) <<  std::fixed << (long double) (finish - start) / CLOCKS_PER_SEC << " seconds." << std::endl;
	else
		std::cout << "Duration    : Less than 0.01 seconds." << std::endl;
}

void MathToolbox::printStatisticsDeprecated (HERO_PTR yah, TIME_DIFF * cputime)
{
	unsigned char pointer;

	yah->print_Name ();
	for (pointer = 1; pointer < (unsigned char) DIFF_SEC_SKILLS; pointer++)
		std::cout << (char *) get_skill_name_12 (pointer) << ": " << std::setprecision (4) << (double) (100.0 * probabilities [pointer]) << " %." << std::endl;
	std::cout << std::endl;

	std::cout << "Duration    : " << cputime->secs << "." << std::setw(6) << std::setfill('0') << cputime->usecs << " seconds." << std::endl;
}

void MathToolbox::printStatisticsGODeprecated (HERO_PTR yah, TIME_DIFF * cputime)
{
	unsigned char pointer;

	yah->print_Name ();
	for (pointer = 0; pointer < (unsigned char) DIFF_SEC_SKILLS-1; pointer++)
		std::cout << (char *) get_skill_name_12 (Skills_Game_Index_to_Alphabetic[pointer]) << ": " << std::setprecision (4) << (double) (100.0 * probabilities [Skills_Game_Index_to_Alphabetic[pointer]]) << " %." << std::endl;
	std::cout << std::endl;

	std::cout << "Duration    : " << cputime->secs << "." << std::setw(6) << std::setfill('0') << cputime->usecs << " seconds." << std::endl;
}

/*****************************************************************************/

void MathToolbox::printStatisticsNeat (HERO_PTR yah, const clock_t start, const clock_t finish)
{
	unsigned char pointer;

	yah->print_Name ();
	for (pointer = 1; pointer < (unsigned char) DIFF_SEC_SKILLS; pointer++)
		std::cout << (char *) get_skill_name_12 (pointer) << ": " << std::setw (8) << std::setprecision (4) << std::fixed << (double) (100.0 * probabilities [pointer]) << " %" << std::endl;
	std::cout << std::endl;

	if ((double) (finish - start) / CLOCKS_PER_SEC >= 0.01)
		std::cout << "Duration    : " << std::setprecision (2) <<  std::fixed << (long double) (finish - start) / CLOCKS_PER_SEC << " seconds." << std::endl;
	else
		std::cout << "Duration    : Less than 0.01 seconds." << std::endl;
}

void MathToolbox::printStatisticsNeat (HERO_PTR yah, TIME_DIFF * cputime)
{
	unsigned char pointer;

	yah->print_Name ();
	for (pointer = 1; pointer < (unsigned char) DIFF_SEC_SKILLS; pointer++)
		std::cout << (char *) get_skill_name_12 (pointer) << ": " << std::setw (3) << (int) occurrences [pointer] << "/255 = " << std::setw (8) << std::setprecision (4) << std::fixed << (double) (100.0 * probabilities [pointer]) << " %" << std::endl;
	std::cout << std::endl;

	std::cout << "Duration    : " << cputime->secs << "." << std::setw(6) << std::setfill('0') << cputime->usecs << " seconds." << std::endl;
}

void MathToolbox::printStatisticsGONeat (HERO_PTR yah, TIME_DIFF * cputime)
{
	unsigned char pointer;

	yah->print_Name ();
	for (pointer = 0; pointer < (unsigned char) DIFF_SEC_SKILLS - 1; pointer++)
		std::cout << (char *) get_skill_name_12 (Skills_Game_Index_to_Alphabetic[pointer]) << ": " << std::setw (3) << (int) occurrences [Skills_Game_Index_to_Alphabetic[pointer]] << "/255 = " << std::setw (8) << std::setprecision (4) << std::fixed << (double) (100.0 * probabilities [Skills_Game_Index_to_Alphabetic[pointer]]) << " %" << std::endl;
	std::cout << std::endl;

	std::cout << "Duration    : " << cputime->secs << "." << std::setw(6) << std::setfill('0') << cputime->usecs << " seconds." << std::endl;
}

/*****************************************************************************/

void MathToolbox::printStatisticsFancy (HERO_PTR yah, const clock_t start, const clock_t finish)
{
	unsigned char	pointer, j, signs;

	yah->print_Name ();
	for (pointer = 1; pointer < (unsigned char) DIFF_SEC_SKILLS; pointer++) {
		std::cout << (char *) get_skill_name_12 (pointer) << " |";
		if ((probabilities[pointer] > 0.0) && (probabilities[pointer] < 0.01))
			std::cout << ">";
		else if (probabilities[pointer] >= 0.01) {
			if (probabilities[pointer] >= 0.03)
				std::cout << "=";
			signs = (unsigned char) ((int) (100.0 * probabilities [pointer] - 1.0)) / 2;
			for (j = 0; j < (signs - 1); j++)
				std::cout << "=";
			std::cout << ">";
		}
		std::cout << "  " << std::setprecision (2) <<  std::fixed << (double) 100.0 * probabilities [pointer] << " %" << std::endl;
	}
	std::cout << std::endl;

	if ((double) (finish - start) / CLOCKS_PER_SEC >= 0.01)
		std::cout << "Duration    : " << std::setprecision (2) <<  std::fixed << (long double) (finish - start) / CLOCKS_PER_SEC << " seconds." << std::endl;
	else
		std::cout << "Duration    : Less than 0.01 seconds." << std::endl;
}

void MathToolbox::printStatisticsFancy (HERO_PTR yah, TIME_DIFF * cputime)
{
	unsigned char	pointer, j, signs;

	yah->print_Name ();
	for (pointer = 1; pointer < (unsigned char) DIFF_SEC_SKILLS; pointer++) {
		std::cout << (char *) get_skill_name_12 (pointer) << " |";
		if ((probabilities[pointer] > 0.0) && (probabilities[pointer] < 0.01))
			std::cout << ">";
		else if (probabilities[pointer] >= 0.01) {
			if (probabilities[pointer] >= 0.03)
				std::cout << "=";
			signs = (unsigned char) ((int) (100.0 * probabilities [pointer] - 1.0)) / 2;
			for (j = 0; j < (signs - 1); j++)
				std::cout << "=";
			std::cout << ">";
		}
		std::cout << "  " << std::setprecision (2) <<  std::fixed << (double) 100.0 * probabilities [pointer] << " %" << std::endl;
	}
	std::cout << std::endl;

	std::cout << "Duration     : " << cputime->secs << "." << std::setw(6) << std::setfill('0') << cputime->usecs << " seconds." << std::endl;
}

void MathToolbox::printStatisticsGOFancy (HERO_PTR yah, TIME_DIFF * cputime)
{
	unsigned char	pointer, j, signs;

	yah->print_Name ();
	for (pointer = 0; pointer < (unsigned char) DIFF_SEC_SKILLS-1; pointer++) {
		std::cout << (char *) get_skill_name_12 (Skills_Game_Index_to_Alphabetic[pointer]) << " |";
		if ((probabilities[Skills_Game_Index_to_Alphabetic[pointer]] > 0.0) && (probabilities[Skills_Game_Index_to_Alphabetic[pointer]] < 0.01))
			std::cout << ">";
		else if (probabilities[Skills_Game_Index_to_Alphabetic[pointer]] >= 0.01) {
			if (probabilities[Skills_Game_Index_to_Alphabetic[pointer]] >= 0.03)
				std::cout << "=";
			signs = (unsigned char) ((int) (100.0 * probabilities [Skills_Game_Index_to_Alphabetic[pointer]] - 1.0)) / 2;
			for (j = 0; j < (signs - 1); j++)
				std::cout << "=";
			std::cout << ">";
		}
		std::cout << "  " << std::setprecision (2) <<  std::fixed << (double) 100.0 * probabilities [Skills_Game_Index_to_Alphabetic[pointer]] << " %" << std::endl;
	}
	std::cout << std::endl;

	std::cout << "Duration     : " << cputime->secs << "." << std::setw(6) << std::setfill('0') << cputime->usecs << " seconds." << std::endl;
}

/*****************************************************************************/
