/*
** Author: dimis
** email : diochnos AT gmail DOT com
**
** File Name: policies.cpp
**
** skills (version 1.0)
** Date Started :  7 September 2009.
** Date Finished: 14 September 2009.
**
** skills (version 2.0)
** Date Started : 15 September 2009.
** Date Finished: 16 September 2009.
**
**
** Tab space: 3
**
**
** The file that contains the implementation of different policies.
**
*/

#include <iostream>
#include "sec_skills_mighty.h"
#include "mighty_classes.h"
#include "policies.h"

DecisionMaker::DecisionMaker(HERO_PTR a_hero)
{
	myHero = a_hero;
}

DecisionMaker::~DecisionMaker() {}


/*
** AlwaysRight.
*/

HeroAlwaysRight::HeroAlwaysRight(HERO_PTR x):DecisionMaker(x)
{
	LoopInitializer ();
}

void HeroAlwaysRight::LoopInitializer () {}

unsigned char HeroAlwaysRight::applyPolicy (SKILL_OFFER_PTR offer)
{
	if (offer->right != NULL_SKILL)
		return 	offer->right;
	else
		return 	offer->left;
}


/*
** AlwaysLeft.
*/

HeroAlwaysLeft::HeroAlwaysLeft(HERO_PTR x):DecisionMaker(x) 
{
	LoopInitializer ();
}

void HeroAlwaysLeft::LoopInitializer () {}

unsigned char HeroAlwaysLeft::applyPolicy (SKILL_OFFER_PTR offer)
{
	if (offer->left != NULL_SKILL)
		return 	offer->left;
	else {
		std::cout << "Problem in your thoughts?" << std::endl;
		return 	NULL_SKILL;
	}
}


/*
** AlwaysLeft Then Preference.
*/

HeroAlwaysLeftThenPref::HeroAlwaysLeftThenPref(HERO_PTR x, unsigned char * selections):DecisionMaker(x)
{
	int i;

	for (i = 0; i < DIFF_SEC_SKILLS; i++)
		Preference[ selections[i] ] = i;

	LoopInitializer ();
}

void HeroAlwaysLeftThenPref::LoopInitializer () {}

unsigned char HeroAlwaysLeftThenPref::applyPolicy (SKILL_OFFER_PTR offer)
{
	if (myHero->get_skill_slot(offer->left) < SKILL_SLOTS) {
		// Skill has already been taken. Therefore we must upgrade according to policy.
		return 	offer->left;
	}
	else if (Preference[offer->left] < Preference[offer->right]) // Both skills are brand new.
		return 	offer->left;
	else // Both skills are brand new.
		return 	offer->right;
}


/*
** Seek for preferred skills otherwise upgrade.
*/

HeroSeekPrefOtherwiseUpgrade::HeroSeekPrefOtherwiseUpgrade(HERO_PTR x, unsigned char * selections):DecisionMaker(x)
{
	int i;

	for (i = 0; i < DIFF_SEC_SKILLS; i++)
		Preference[ selections[i] ] = i;

	LoopInitializer ();

	//printPreferences ();
	return;
}

void HeroSeekPrefOtherwiseUpgrade::LoopInitializer ()
{
	unwanted = 0;
}

unsigned char HeroSeekPrefOtherwiseUpgrade::applyPolicy (SKILL_OFFER_PTR offer)
{
	if ((myHero->get_skill_slot(offer->left) < SKILL_SLOTS) && (myHero->get_skill_slot(offer->right) < SKILL_SLOTS)) {
		// Both skills have already been taken. Choose the one with higher preference.
		if (Preference[offer->left] < Preference[offer->right])
			return 	offer->left;
		else
			return 	offer->right;
	}
	else if (myHero->get_skill_slot(offer->left) < SKILL_SLOTS) {
		// Left Skill ONLY has already been taken.
		//std::cout << (int) Preference[offer->right] << " (" << (int) (SKILL_SLOTS - unwanted) << ")." << std::endl;
		if ( qualifies_on_top (offer->right) )
			return 	offer->right; //Get new skill. We are sure that it's on TOP of our list AT THE MOMENT of selection. ;)
		else // offer->right is not so high on our preference list. Hence we upgrade the left one.
			return 	offer->left; // Upgrade existing skill.
	}
	else { 
		// Both skills are brand new. Pick the one you prefer more.
		if (Preference[offer->left] < Preference[offer->right]) {
			if ( !qualifies_on_top(offer->left) )
				unwanted++;
			return 	offer->left;
		}
		else {
			if ( !qualifies_on_top(offer->right) )
				unwanted++;
			return 	offer->right;
		}
	}
}

unsigned char HeroSeekPrefOtherwiseUpgrade::get_number_of_unwanted_skills () const
{
	return unwanted;
}

void 	HeroSeekPrefOtherwiseUpgrade::printPreferences ()
{
	std::cout << "\n\nPreferences [SPOU]" << std::endl;
	for (int i = 0; i < DIFF_SEC_SKILLS; i++) {
		std::cout << get_skill_name_12 ( i ) << ": " << (int) Preference[i] << std::endl;
	}
}

bool	HeroSeekPrefOtherwiseUpgrade::qualifies_on_top (const unsigned char skill) const
{
	if (Preference [skill] >= SKILL_SLOTS)
		return false;

	int i, counter;
	unsigned char sorted_pref [DIFF_SEC_SKILLS];

	for (i = 0; i < DIFF_SEC_SKILLS; i++)
		sorted_pref [ Preference [i] ] = i;

	counter = get_number_of_unwanted_skills ();
	for (i = SKILL_SLOTS - 1; i >= 0; i--) {
		if ( counter == 0 )
			break;
		if ( sorted_pref [i] == skill )
			break;
		if ( myHero->get_skill_slot ( sorted_pref [i] ) >= SKILL_SLOTS ) {
			// Out of range ==> Hero does not have this skill.
			// Moreover, this skill has lower 'priority' than given skill.
			counter--;
		}
	}

	return	(counter == 0) ? true : false;
}


/*
** Follow Frank's policy.
*/

HeroFollowsFrank::HeroFollowsFrank(HERO_PTR x, unsigned char * selections):DecisionMaker(x)
{
	int i;
	unsigned char temp;

	for (i = 0; i < DIFF_SEC_SKILLS; i++)
		Preference[ selections[i] ] = i;

	initial_available_upgrades = 0;
	for (i = 0; i < SKILL_SLOTS; i++) {
		if (x->get_skill_from_slot (i) != NULL_SKILL) {
			temp = x->get_skill_expertize_on_slot (i);
			initial_available_upgrades += 3 - temp;
		}
	}

	LoopInitializer ();

	//printPreferences ();
	return;
}

void HeroFollowsFrank::LoopInitializer ()
{
	unwanted = 0;
	available_upgrades = initial_available_upgrades;
}

unsigned char HeroFollowsFrank::applyPolicy (SKILL_OFFER_PTR offer)
{
	unsigned char selection;

	if (available_upgrades > 1) {
		if ((myHero->get_skill_slot(offer->left) < SKILL_SLOTS) && (myHero->get_skill_slot(offer->right) < SKILL_SLOTS)) {
			// Both skills have already been taken. Upgrade the one with higher preference.
			selection = (Preference[offer->left] < Preference[offer->right]) ? offer->left : offer->right;
		}
		else {
			// Left Skill ONLY has already been taken.
			selection = offer->left;
		}

		available_upgrades--;
		return selection;
	}
	else if (available_upgrades == 0) {
		// Both skills are brand new. Pick the one with higher preference.
		// This needs a check whether or not you pick something 'unwanted'.
		selection = (Preference[offer->left] < Preference[offer->right]) ? offer->left : offer->right;
		if ( !qualifies_on_top(selection) )
			unwanted++;
		available_upgrades = 2;
		return selection;
	}

	// Now available_upgrades is equal to 1. Check if something good was offered.
	if (offer->right != NULL_SKILL) {
		if ( qualifies_on_top (offer->right) ) {
			// User wants this skill among the 8. So pick it.
			available_upgrades += 2;
			return offer->right;
		}
		else {
			// Upgrade existing skill.
			available_upgrades--;
			return offer->left;
		}
	}
	else {
		// Final upgrade for hero.
		available_upgrades--;
		return offer->left;
	}
}

unsigned char HeroFollowsFrank::get_number_of_unwanted_skills () const
{
	return unwanted;
}

void 	HeroFollowsFrank::printPreferences ()
{
	std::cout << "\n\nPreferences [FRANK]" << std::endl;
	for (int i = 0; i < DIFF_SEC_SKILLS; i++) {
		std::cout << get_skill_name_12 ( i ) << ": " << (int) Preference[i] << std::endl;
	}
}

bool	HeroFollowsFrank::qualifies_on_top (const unsigned char skill) const
{
	if (Preference [skill] >= SKILL_SLOTS)
		return false;

	int i, counter;
	unsigned char sorted_pref [DIFF_SEC_SKILLS];

	for (i = 0; i < DIFF_SEC_SKILLS; i++)
		sorted_pref [ Preference [i] ] = i;

	counter = get_number_of_unwanted_skills ();
	for (i = SKILL_SLOTS - 1; i >= 0; i--) {
		if ( counter == 0 )
			break;
		if ( sorted_pref [i] == skill )
			break;
		if ( myHero->get_skill_slot ( sorted_pref [i] ) >= SKILL_SLOTS ) {
			// Out of range ==> Hero does not have this skill.
			// Moreover, this skill has lower 'priority' than given skill.
			counter--;
		}
	}

	return	(counter == 0) ? true : false;
}

/*
** always accept New Magic offers; Then follow Preferences (NMTP).
** 
*/
/*
HeroNMTP::HeroNMTP(HERO_PTR x, unsigned char * selections):DecisionMaker(x)
{
	int i;

	for (i = 0; i < DIFF_SEC_SKILLS; i++)
		Preference[ selections[i] ] = i;

	LoopInitializer ();

	//printPreferences ();
	return;
}

void HeroNMTP::LoopInitializer ()
{
	unwanted = 0;
}

unsigned char HeroNMTP::applyPolicy (SKILL_OFFER_PTR offer)
{
	unsigned char selection;

	if (((offer->right == AIR_MAGIC) || (offer->right == EARTH_MAGIC) || (offer->right == FIRE_MAGIC) || (offer->right == WATER_MAGIC)) 
		&& (myHero->get_skill_slot (offer->right) >= SKILL_SLOTS)) {
		// This is a brand new Magic School offer. Take it!
		return offer->right;
	}

}

unsigned char HeroNMTP::get_number_of_unwanted_skills () const
{
	return unwanted;
}

void 	HeroNMTP::printPreferences ()
{
	std::cout << "\n\nPreferences [FRANK]" << std::endl;
	for (int i = 0; i < DIFF_SEC_SKILLS; i++) {
		std::cout << get_skill_name_12 ( i ) << ": " << (int) Preference[i] << std::endl;
	}
}

bool	HeroNMTP::qualifies_on_top (const unsigned char skill) const
{
	if (Preference [skill] >= SKILL_SLOTS)
		return false;

	int i, counter;
	unsigned char sorted_pref [DIFF_SEC_SKILLS];

	for (i = 0; i < DIFF_SEC_SKILLS; i++)
		sorted_pref [ Preference [i] ] = i;

	counter = get_number_of_unwanted_skills ();
	for (i = SKILL_SLOTS - 1; i >= 0; i--) {
		if ( counter == 0 )
			break;
		if ( sorted_pref [i] == skill )
			break;
		if ( myHero->get_skill_slot ( sorted_pref [i] ) >= SKILL_SLOTS ) {
			// Out of range ==> Hero does not have this skill.
			// Moreover, this skill has lower 'priority' than given skill.
			counter--;
		}
	}

	return	(counter == 0) ? true : false;
}
*/

